MODULE struct
! Georg Madsen 2002
  CHARACTER(80)             :: title
  CHARACTER(4)              :: lattic,irel
  REAL(8)                   :: aa,bb,cc,pia(3),alpha(3)

  INTEGER                  :: nat,ndif
  INTEGER,ALLOCATABLE      :: mult(:),jrj(:),iatnr(:),isplit(:)
  REAL(8),ALLOCATABLE       :: r0(:),dx(:),rmt(:),zz(:)
  CHARACTER(10),ALLOCATABLE :: aname(:)
  REAL(8),POINTER           :: pos(:,:)

  INTEGER                  :: nsym,iord
  INTEGER,POINTER          :: iz(:,:,:),inum(:)
  REAL(8),POINTER           :: tau(:,:)

!     ROTLOC(3,3,JATOM) 
!     ROTATE FROM THE GENERAL COORDINATION-SYSTEM INTO 
!     LOCAL SYSTEMS WITH SPECIFIED POINTSYMMETRY.
  REAL(8),ALLOCATABLE       :: rotloc(:,:,:)


CONTAINS
  SUBROUTINE init_struct
    USE reallocate
    USE defs
    IMPLICIT NONE

    INTEGER                :: ios
    REAL(8)                 :: test0

    INTEGER                :: index,i,j,j1,j2,m,jatom

    test0=1.D-5

    read (20,1000) title
    read (20,1010) lattic,nat,irel
    ALLOCATE(aname(nat),mult(0:nat),jrj(nat),r0(nat),dx(nat),rmt(nat),zz(nat),rotloc(3,3,nat),iatnr(nat),isplit(nat))
    ALLOCATE (pos(3,48*nat))
    mult(0)=0
    read (20,1020) aa,bb,cc,alpha(1),alpha(2),alpha(3)
    IF(ABS(ALPHA(1)).LT.test0) ALPHA(1)=ninety
    IF(ABS(ALPHA(2)).LT.test0) ALPHA(2)=ninety
    IF(ABS(ALPHA(3)).LT.test0) ALPHA(3)=ninety
    INDEX=0
    DO jatom=1,NAT
       INDEX=INDEX+1
       READ(20,1030,iostat=ios) iatnr(jatom),( pos(j,index),j=1,3 ), &
            mult(jatom),isplit(jatom) 
       IF(ios /= 0) THEN
          WRITE(6,*) iatnr(jatom),( pos(j,index),j=1,3 ), &
               mult(jatom),isplit(jatom) 
          WRITE(6,*) 'ERROR IN STRUCT FILE READ'
          STOP
       ENDIF
       IF (mult(jatom) .EQ. 0) THEN
          WRITE (6,6000) jatom, index, mult(jatom)
          STOP
       ENDIF
       DO m=1,mult(jatom)-1                                     
          index=index+1                                            
          READ(20,1031) iatnr(jatom),( pos(j,index),j=1,3)         
       ENDDO
       READ(20,1050) aname(jatom),jrj(jatom),r0(jatom),rmt(jatom), &
            zz(jatom)
       dx(jatom)=LOG(rmt(jatom)/r0(jatom)) / (jrj(jatom)-1)           
       rmt(jatom)=r0(jatom)*EXP( dx(jatom)*(jrj(jatom)-1) )           
       READ(20,1051) ((rotloc(i,j,jatom),i=1,3),j=1,3)                
    ENDDO
    ndif=index
    CALL doreallocate(pos, 3, ndif)
    READ(20,1151) iord
    nsym=iord
    ALLOCATE(iz(3,3,nsym),tau(3,nsym),inum(nsym))
    DO j=1,iord
       READ(20,1101) ( (iz(j1,j2,j),j1=1,3),tau(j2,j),j2=1,3 ),inum(j)
    ENDDO
  
1000 FORMAT(A80)                                                       
1010 FORMAT(A4,24X,I2,/,13X,A4)                                 
1020 FORMAT(6F10.7,10X,F10.7)                                          
1030 FORMAT(5X,I3,4X,F10.7,3X,F10.7,3X,F10.7,/,15X,I2,17X,I2)          
1031 FORMAT(5X,I3,4X,F10.7,3X,F10.7,3X,F10.7)                          
1050 FORMAT(A10,5X,I5,5X,F10.9,5X,F10.5,5X,F10.5)                      
1051 FORMAT(20X,3F10.8)                                             
1101 FORMAT(3(3I2,F10.8/),I8)
1151 FORMAT(I4)
6000 FORMAT(///,3X,'ERROR IN READING STRUCT : MULT(JATOM)=0 ...', &
          /, 20X,'JATOM=',I3,3X,'INDEX=',I3,3X,'MULT=',I3)
  END SUBROUTINE init_struct


  SUBROUTINE invert_struct(mat1,inv_mat1)
    REAL(8)        :: mat1(3,3),inv_mat1(3,3)
    inv_mat1(1,1)=mat1(2,2)*mat1(3,3)-mat1(3,2)*mat1(2,3)                 
    inv_mat1(2,1)=mat1(3,2)*mat1(1,3)-mat1(1,2)*mat1(3,3)                 
    inv_mat1(3,1)=mat1(1,2)*mat1(2,3)-mat1(2,2)*mat1(1,3)                 
    inv_mat1(1,2)=mat1(2,3)*mat1(3,1)-mat1(3,3)*mat1(2,1)                 
    inv_mat1(2,2)=mat1(3,3)*mat1(1,1)-mat1(1,3)*mat1(3,1)                 
    inv_mat1(3,2)=mat1(1,3)*mat1(2,1)-mat1(2,3)*mat1(1,1)                 
    inv_mat1(1,3)=mat1(2,1)*mat1(3,2)-mat1(3,1)*mat1(2,2)                 
    inv_mat1(2,3)=mat1(3,1)*mat1(1,2)-mat1(1,1)*mat1(3,2)                 
    inv_mat1(3,3)=mat1(1,1)*mat1(2,2)-mat1(2,1)*mat1(1,2)                 
  END SUBROUTINE invert_struct

END MODULE struct

!!$  REAL(8) FUNCTION dGd_struct(vec,mat)
!!$    USE defs
!!$    REAL(8),INTENT(IN)      :: vec(3),mat(3,3)
!!$    INTEGER                 :: i,j
!!$    REAL(8)                 :: vecdum(3)
!!$    vecdum = ZERO
!!$    DO i=1,3
!!$       DO j=1,3
!!$          vecdum(i)=vecdum(i) + mat(i,j)*vec(j)
!!$       ENDDO
!!$    ENDDO
!!$    dGd_struct = ZERO
!!$    DO i=1,3
!!$       dGd_struct = dGd_struct + vecdum(i)*vec(i)
!!$    ENDDO
!!$  END FUNCTION dGd_struct

