MODULE struct
! Georg Madsen 2002
  CHARACTER(80)             :: title
  CHARACTER(4)              :: lattic,irel
  REAL(8)                   :: aa,bb,cc,pia(3),alpha(3)
  REAL(8)                   :: volume

  INTEGER                  :: nat,ndif
  INTEGER,POINTER          :: mult(:),jrj(:),iatnr(:),isplit(:)
  REAL(8),POINTER          :: r0(:),dx(:),rmt(:),zz(:)
  CHARACTER(10),POINTER    :: aname(:)
  REAL(8),POINTER           :: pos(:,:)

  INTEGER                  :: nsym,iord
  INTEGER,POINTER          :: iz(:,:,:),inum(:)
  REAL(8),POINTER          :: tau(:,:)

!     ROTATE FROM THE GENERAL COORDINATION-SYSTEM INTO 
!     LOCAL SYSTEMS WITH SPECIFIED POINTSYMMETRY.
  REAL(8),ALLOCATABLE       :: rotloc(:,:,:)

CONTAINS
  SUBROUTINE init_struct
    USE reallocate
    USE defs
    IMPLICIT NONE

    INTEGER                :: ios
    REAL(8)                 :: test0

    INTEGER                :: index,i,j,j1,j2,m,jatom

    test0=1.D-5

    read (20,1000) title
    read (20,1010) lattic,nat,irel
    ALLOCATE(aname(nat),mult(0:nat),jrj(nat),r0(nat),dx(nat),rmt(nat),zz(nat),rotloc(3,3,nat),iatnr(nat),isplit(nat))
    ALLOCATE (pos(3,48*nat))
    mult(0)=0
    read (20,1020) aa,bb,cc,alpha(1),alpha(2),alpha(3)
    IF(ABS(ALPHA(1)).LT.test0) ALPHA(1)=ninety
    IF(ABS(ALPHA(2)).LT.test0) ALPHA(2)=ninety
    IF(ABS(ALPHA(3)).LT.test0) ALPHA(3)=ninety
    INDEX=0
    DO jatom=1,NAT
       INDEX=INDEX+1
       READ(20,1030,iostat=ios) iatnr(jatom),( pos(j,index),j=1,3 ), &
            mult(jatom),isplit(jatom) 
       IF(ios /= 0) THEN
          WRITE(6,*) iatnr(jatom),( pos(j,index),j=1,3 ), &
               mult(jatom),isplit(jatom) 
          WRITE(6,*) 'ERROR IN STRUCT FILE READ'
          STOP
       ENDIF
       IF (mult(jatom) .EQ. 0) THEN
          WRITE (6,6000) jatom, index, mult(jatom)
          STOP
       ENDIF
       DO m=1,mult(jatom)-1                                     
          index=index+1                                            
          READ(20,1031) iatnr(jatom),( pos(j,index),j=1,3)         
       ENDDO
       READ(20,1050) aname(jatom),jrj(jatom),r0(jatom),rmt(jatom), &
            zz(jatom)
       dx(jatom)=LOG(rmt(jatom)/r0(jatom)) / (jrj(jatom)-1)           
       rmt(jatom)=r0(jatom)*EXP( dx(jatom)*(jrj(jatom)-1) )           
       READ(20,1051) ((rotloc(i,j,jatom),i=1,3),j=1,3)                
    ENDDO
    ndif=index
    CALL doreallocate(pos, 3, ndif)
    READ(20,1151) iord
    nsym=iord
    ALLOCATE(iz(3,3,nsym),tau(3,nsym),inum(nsym))
    DO j=1,iord
       READ(20,1101) ( (iz(j1,j2,j),j1=1,3),tau(j2,j),j2=1,3 ),inum(j)
    ENDDO

1000 FORMAT(A80)                                                       
1010 FORMAT(A4,24X,I2,/,13X,A4)                                 
1020 FORMAT(6F10.7,10X,F10.7)                                          
1030 FORMAT(5X,I3,4X,F10.7,3X,F10.7,3X,F10.7,/,15X,I2,17X,I2)          
1031 FORMAT(5X,I3,4X,F10.7,3X,F10.7,3X,F10.7)                          
1050 FORMAT(A10,5X,I5,5X,F10.9,5X,F10.5,5X,F10.5)                      
1051 FORMAT(20X,3F10.8)                                             
1101 FORMAT(3(3I2,F10.8/),I8)
1151 FORMAT(I4)
6000 FORMAT(///,3X,'ERROR IN READING STRUCT : MULT(JATOM)=0 ...', &
         /, 20X,'JATOM=',I3,3X,'INDEX=',I3,3X,'MULT=',I3)
  END SUBROUTINE init_struct

  SUBROUTINE write_struct
    USE defs
    IMPLICIT NONE

    INTEGER                :: index,i,j,j1,j2,m,jatom

    WRITE(20,1000) title
    WRITE(20,1010) lattic,nat,irel
    WRITE(20,1020) aa,bb,cc,alpha(1),alpha(2),alpha(3)
    INDEX=0
    DO jatom=1,NAT
       INDEX=INDEX+1
       WRITE(20,1030) iatnr(jatom),( pos(j,index),j=1,3 ), &
            mult(jatom),isplit(jatom) 
       DO m=1,mult(jatom)-1                                     
          index=index+1                                            
          WRITE(20,1031) iatnr(jatom),( pos(j,index),j=1,3)         
       ENDDO
       WRITE(20,1050) aname(jatom),jrj(jatom),r0(jatom),rmt(jatom), &
            zz(jatom)
       dx(jatom)=LOG(rmt(jatom)/r0(jatom)) / (jrj(jatom)-1)           
       rmt(jatom)=r0(jatom)*EXP( dx(jatom)*(jrj(jatom)-1) )           
       WRITE(20,1051) ((rotloc(i,j,jatom),i=1,1),j=1,3)                
       WRITE(20,1052) ((rotloc(i,j,jatom),i=1,3),j=2,3)                
    ENDDO
    ndif=index
    WRITE(20,1151) nsym
    DO j=1,nsym
       WRITE(20,1101) ( (iz(j1,j2,j),j1=1,3),tau(j2,j),j2=1,3 ),inum(j)
    ENDDO

1000 FORMAT(A80)                                                       
1010 FORMAT(A4,'LATTICE,NONEQUIV.ATOMS:',I3,/,'MODE OF CALC=',A4)
1020 FORMAT(6F10.6,10X,F10.6)                                          
1030 FORMAT('ATOM',1X,I3,': X=',F10.8,1X,'Y=',F10.8,1X,'Z=',F10.8,/,10X, & 
         'MULT=',I2,10X,'ISPLIT=',I2)
1031 FORMAT(5X,I3,': X=',F10.8,1X,'Y=',F10.8,1X,'Z=',F10.8)                          
1050 FORMAT(A10,1X,'NPT=',I5,2X,'R0=',F10.9,1X,'RMT=',F10.5,3X,'Z:',F5.0)                      
1051 FORMAT('LOCAL ROT MATRIX:',4X,F9.7,1X,F9.7,1X,F9.7)
1052 FORMAT(21X,F9.7,1X,F9.7,1X,F9.7)
1101 FORMAT(3(3I2,1X,F10.8/),I8)
1151 FORMAT(I4,6X,'NUMBER OF SYMMETRY OPERATIONS')
  END SUBROUTINE write_struct

  SUBROUTINE newnam_struct(acell,lattice)
    CHARACTER(1) :: lattice
    REAL(8)      :: acell(3)
    acell(1)=aa; acell(2)=bb; acell(3)=cc
    SELECT CASE (LATTIC(1:1))
    CASE ('H','P')       !.....Primitive lattices
       lattice='P'
    CASE ( 'F')
       lattice='F'
    CASE ( 'B' )
       lattice='I'
    CASE( 'C' )
       SELECT CASE (lattic(2:3))
       CASE ( 'XY' )
          lattice='C'
       CASE( 'XZ ' )
          lattice='B'
       CASE( 'YZ' )
          lattice='A'
       END SELECT
    CASE ( 'R' )
       STOP 'SPECIAL TREATMENT NOT IMPLEMETED'
    CASE DEFAULT
       STOP 'Wrong lattice'
    END SELECT
  END SUBROUTINE newnam_struct

  SUBROUTINE breaksym_struct(lattice)
    USE defs
    CHARACTER(1)    :: lattice
    INTEGER,POINTER :: jrj_2(:)
    REAL(8),POINTER :: r0_2(:),dx_2(:),rmt_2(:),zz_2(:)
    CHARACTER(10),POINTER :: aname_2(:)
    IF(lattice/='P') STOP 'Nonprim cell'
    nsym = 1
    DEALLOCATE(iz,tau,inum)
    ALLOCATE(iz(3,3,1),tau(3,1),inum(1))
    iz(:,:,1)=iunity
    tau=ZERO
    inum=1
    DEALLOCATE(iatnr,isplit,rotloc)
    ALLOCATE(iatnr(ndif),isplit(ndif),rotloc(3,3,ndif))
    ALLOCATE(jrj_2(ndif),r0_2(ndif),dx_2(ndif),rmt_2(ndif),zz_2(ndif),aname_2(ndif))
    index=0
    DO jneq=1,nat
       DO jeq=1,mult(jneq)
          index=index+1
          jrj_2(index)  = jrj(jneq)
          r0_2(index)   = r0(jneq)
          dx_2(index)   = dx(jneq)
          rmt_2(index)  = rmt(jneq)
          zz_2(index)   = zz(jneq)
          aname_2(index)= aname(jneq)
          iatnr(index)  = -index
          rotloc(:,:,index)=unity
       ENDDO
    ENDDO
    DEALLOCATE(jrj,r0,dx,rmt,zz,aname)
    jrj=>jrj_2; r0=>r0_2; dx=>dx_2; rmt=>rmt_2; zz=>zz_2; aname=>aname_2 
    nat=ndif
    DEALLOCATE(mult)
    ALLOCATE(mult(nat))
    mult=1
    isplit = 1
  END SUBROUTINE breaksym_struct

  SUBROUTINE supercell_struct(ncell,lattice)
    IMPLICIT NONE
    CHARACTER(1)          :: lattice
    INTEGER,INTENT(IN)    :: ncell(3)
    REAL(8),POINTER       :: pos2(:,:)
    INTEGER               :: i1,i2,i3,na,index
    ALLOCATE(pos2(3,ndif*ncell(1)*ncell(2)*ncell(3)))
    index=0
    DO na = 1, ndif
       pos(1,na) = pos(1,na) / REAL(ncell(1))
       pos(2,na) = pos(2,na) / REAL(ncell(2))
       pos(3,na) = pos(3,na) / REAL(ncell(3))
       DO i3 = 0, ncell(3)-1
          DO i2 = 0, ncell(2)-1
             DO i1 = 0, ncell(1)-1
                index=index+1
                pos2(1,index) =pos(1,na)+ real(i1)/real(ncell(1)) 
                pos2(2,index) =pos(2,na)+ real(i2)/real(ncell(2))
                pos2(3,index) =pos(3,na)+ real(i3)/real(ncell(3))
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    DEALLOCATE(pos)
    pos => pos2
    nat=ndif
    ndif=index
    mult=ncell(1)*ncell(2)*ncell(3)
    call breaksym_struct(lattice)
    aa = aa*ncell(1)
    bb = bb*ncell(2)
    cc = cc*ncell(3)
  END SUBROUTINE supercell_struct


END MODULE struct
